import logging

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_view import Documents_view
from kaf_pas.kd.models.uploades import Uploades

logger = logging.getLogger(__name__)


class Uploades_documentsQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Uploades_documentsManager(AuditManager):

    @staticmethod
    def getRecord(record):
        document_view = record.document_view
        res = {
            'id': record.id,
            'document_id': record.document.id,
            'document__path_id': record.document.path.id,
            'document__attr_type_id': record.document.attr_type.id,
            'document__attr_type__code': record.document.attr_type.code,
            'document__attr_type__name': record.document.attr_type.name,
            'document__file_document': record.document.file_document,
            'document__file_size': record.document.file_size,
            'document__lastmodified': record.document.lastmodified,
            'document__file_modification_time': record.document.file_modification_time,
            'document__file_access_time': record.document.file_access_time,
            'document__file_change_time': record.document.file_change_time,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Uploades_documentsQuerySet(self.model, using=self._db)


class Uploades_documents(AuditModel):
    upload = ForeignKeyCascade(Uploades)
    document = ForeignKeyCascade(Documents)

    objects = Uploades_documentsManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        unique_together = (('upload', 'document'),)



import logging

from bitfield import BitField
from django.db.models import TextField

from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Document_attrsQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def get_attr(self, document, code):
        res = super().filter(document=document, attr_type__code=code)
        _len = len(res)
        if _len == 1:
            return res[0]
        elif _len > 1:
            logger.warning(f'document: {document} has attr:{code} {_len}')
            return res[0]
        else:
            return None


class Document_attrsManager(CommonManagetWithLookUpFieldsManager):
    def get_queryset(self):
        return Document_attrsQuerySet(self.model, using=self._db)

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "attr_type_id": record.attr_type.id,
            "attr_type__code": record.attr_type.code,
            "attr_type__name": record.attr_type.name,
            "attr_type__description": record.attr_type.description if record.attr_type else None,
            "section": record.section,
            "subsection": record.subsection,
            "value_str": record.value_str,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res


class Document_attrs(AuditModel):
    # DEPRECATED !!!!!!!!!!!!!!!!!

    document = ForeignKeyCascade(Documents, verbose_name='Документ')
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип атрибута')

    section = CodeField(verbose_name="Раздел", null=True, blank=True)
    subsection = CodeField(verbose_name="Подраздел", null=True, blank=True)
    value_str = TextField(verbose_name="Значение атрибута", db_index=True)

    props = BitField(flags=(
        ('relevant', 'Актуальность'),
        ('copied', 'Копированный')
    ), default=1, db_index=True)

    objects = Document_attrsManager()

    def __str__(self):
        return self.value_str

    class Meta:
        verbose_name = 'Аттрибуты докуменнта'
        unique_together = (('document', 'attr_type', 'section', 'subsection'),)



# Generated by Django 2.2.1 on 2019-05-24 13:51

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0057_auto_20190524_1351'),
    ]

    operations = [
        migrations.AlterField(
            model_name='materials',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 10:01

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0040_item_line_view'),
        ('production', '0011_temp_production_operationsplan'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operation_plan_status',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField()),
                ('name', isc_common.fields.name_field.NameField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
            ],
            options={
                'verbose_name': 'Статусы',
            },
        ),
        migrations.AddField(
            model_name='operationsplan',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
        migrations.AddField(
            model_name='operationsplan',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AddField(
            model_name='operationsplan',
            name='startdate',
            field=models.DateTimeField(db_index=True, default=None),
        ),
        migrations.AddField(
            model_name='operationsplan',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_types'),
        ),
        migrations.AddField(
            model_name='operationsplan',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_status'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-24 12:18

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0093_status_launch'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='status_launch',
            options={'verbose_name': 'Статусы запусков'},
        ),
        migrations.CreateModel(
            name='Launches',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('date', models.DateTimeField()),
                ('qty', models.PositiveIntegerField()),
                ('demand', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Demand')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='sales.Launches')),
                ('status', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Status_launch')),
            ],
            options={
                'verbose_name': 'Запуски',
            },
        ),
    ]



import logging
import uuid

from django.conf import settings
from django.db import transaction, connection
from django.forms import model_to_dict

from isc_common.datetime import DateToStr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
from isc_common.ws.progressStack import ProgressStack
from kaf_pas.planing.models.operation_types import Operation_types
from kaf_pas.planing.models.status_operation_types import Status_operation_types
from kaf_pas.production.models.launch_operation_material import Launch_operations_material
from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
from kaf_pas.production.models.launch_operations_item import Launch_operations_item

logger = logging.getLogger(__name__)


class OperationsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class OperationsManager(AuditManager):

    @staticmethod
    def make_routing(data):
        from isc_common.auth.models.user import User
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_material import Operation_material
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_resources import Operation_resources
        from kaf_pas.planing.models.operation_standard_prod import Operation_standard_prod
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.planing.models.operation_locations import Operation_locations
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.planing.models.operation_level import Operation_level

        logger.debug(f'data: {data}')

        OperationsManager.clean_routing(data=data)

        launch_id = data.get('id')

        launch = Launches.objects.get(id=launch_id)

        user = User.objects.get(id=data.get('user_id'))

        progress = ProgressStack(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
        )

        id_progress1 = f'launch_{launch.id}_{user.id}'

        demand_str = f'<h3>Подготовка данных для Маршрутизации/Расписания : Запуск № {launch.code} от {DateToStr(launch.date)}</h3>'

        cnt = 0
        mat_view_name = f'tmp_{str(uuid.uuid4()).replace("-", "_")}'
        items = []

        sql_txt = f'''CREATE MATERIALIZED VIEW public.{mat_view_name}
                    TABLESPACE informica
                    AS 
                    with recursive r as (
                                            select *,
                                                   1 as level
                                            from production_launch_item_refs
                                            where parent_id is null
                                              and launch_id = {launch_id}
                                            union all
                                            select production_launch_item_refs.*,
                                                   r.level + 1 as level
                                            from production_launch_item_refs
                                                     join r
                                                          on
                                                              production_launch_item_refs.parent_id = r.child_id)
                                        
                                        select r1.id,
                                               r1.parent_id,
                                               r1.child_id,
                                               r1.launch_id,
                                               r1.stmp1,
                                               r1.stmp2,
                                               coalesce(r1.qty, 0) qty,
                                               r1.level
                                        from (select distinct r.id,
                                                              r.parent_id,
                                                              r.child_id,
                                                              stmp1.value_str                         stmp1,
                                                              stmp2.value_str                         stmp2,
                                                              r.launch_id,
                                                              (
                                                                  select coalesce(kda.value_str::numeric(19, 4), 0)
                                                                  from production_launch_item_line plil
                                                                           join kd_document_attributes kda on kda.id = plil."SPC_CLM_COUNT_id"
                                                                  where plil.child_id = r.child_id
                                                                    and plil.parent_id = r.parent_id
                                                                    and plil.launch_id = r.launch_id) qty,
                                                              (
                                                                  select plil.section
                                                                  from production_launch_item_line plil
                                                                  where plil.child_id = r.child_id
                                                                    and plil.parent_id = r.parent_id
                                                                    and plil.launch_id = r.launch_id) section,
                                                              level
                                              from r
                                                       join ckk_item ci on ci.id = r.child_id
                                                       join kd_document_attributes stmp1 on stmp1.id = ci."STMP_1_id"
                                                       join kd_document_attributes stmp2 on stmp2.id = ci."STMP_2_id"
                                              where r.launch_id = {launch_id}
                                              order by level desc) r1
                                        where lower(r1.section) != 'документация'
                                 WITH DATA;'''

        with transaction.atomic():
            with connection.cursor() as cursor:
                status, _ = Status_operation_types.objects.get_or_create(
                    opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK,
                    code='none',
                    defaults=dict(name='не определен', deliting=False, editing=False)
                )

                cursor.execute(sql_txt)

                cursor.execute(f'select count(*) from {mat_view_name}')
                count, = cursor.fetchone()

                logger.debug(f'{mat_view_name} count(*): {count}')

                progress.show(
                    title=f'<b>Обработано товарных позиций</b>',
                    label_contents=demand_str,
                    cntAll=count * 2,
                    id=id_progress1
                )

                cursor.execute(f'select * from {mat_view_name} order by level desc')
                rows = cursor.fetchall()
                for row in rows:
                    id, parent_id, child_id, launch_id, stmp1, stmp2, qty, level = row

                    if len([item for item in items if item.get('item_id') == child_id]) == 0:
                        operation_income = None
                        cnt1 = Launch_operations_item.objects.filter(item_id=child_id, launch_id=launch_id).count()
                        if cnt1 > 0:
                            first_operation = None
                            for launch_operations_item in Launch_operations_item.objects.filter(item_id=child_id, launch_id=launch_id).order_by('num'):
                                operation_outcome = Operations.objects.create(
                                    opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK,
                                    status=status
                                )

                                Operation_launches.objects.create(operation=operation_outcome, launch=launch)
                                Operation_level.objects.create(operation=operation_outcome, level=level)

                                if operation_income == None:
                                    first_operation = operation_outcome

                                Operation_item.objects.get_or_create(operation=operation_outcome, item=launch_operations_item.item)

                                for launch_operation_material in Launch_operations_material.objects.filter(launch_operationitem=launch_operations_item):
                                    if launch_operation_material.material_askon != None:
                                        Operation_material.objects.get_or_create(operation=operation_outcome, material=launch_operation_material.material_askon)
                                    if launch_operation_material.material != None:
                                        Operation_standard_prod.objects.get_or_create(operation=operation_outcome, material=launch_operation_material.material)

                                if Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item) == 0:
                                    raise Exception(f'Для : {stmp1} : {stmp2} ID={child_id} не задан ресурс.')

                                for launch_operation_resources in Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item):
                                    if launch_operation_resources.resource != None:
                                        Operation_resources.objects.get_or_create(operation=operation_outcome, resource=launch_operation_resources.resource)
                                    if launch_operation_resources.location != None:
                                        Operation_locations.objects.get_or_create(operation=operation_outcome, location=launch_operation_resources.location)

                                Operation_refs.objects.get_or_create(income=operation_income, outcome=operation_outcome)
                                operation_income = operation_outcome
                                cnt1 -= 1
                                if cnt1 == 0:
                                    items.append(dict(item_id=child_id, first_operation=first_operation, last_operation=operation_outcome))
                        else:
                            raise Exception(f'Для : {stmp1} : {stmp2} ID={child_id} не задано ни одной операции.')

                    progress.setCntDone(cnt=cnt, id=id_progress1)
                    cnt += 1

                for row in rows:
                    id, parent_id, child_id, launch_id, stmp1, stmp2, qty, level = row

                    if len([item for item in items if item.get('item_id') == parent_id]) == 0:

                        operation_income = None
                        cnt1 = Launch_operations_item.objects.filter(item_id=parent_id, launch_id=launch_id).count()
                        if cnt1 > 0:
                            first_operation = None
                            for launch_operations_item in Launch_operations_item.objects.filter(item_id=parent_id, launch_id=launch_id).order_by('num'):
                                operation_outcome = Operations.objects.create(
                                    opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK,
                                    status=status
                                )

                                Operation_launches.objects.create(operation=operation_outcome, launch=launch)

                                if operation_income == None:
                                    first_operation = operation_outcome

                                Operation_item.objects.get_or_create(operation=operation_outcome, item=launch_operations_item.item)
                                Operation_level.objects.create(operation=operation_outcome, level=level)

                                for launch_operation_material in Launch_operations_material.objects.filter(launch_operationitem=launch_operations_item):
                                    if launch_operation_material.material_askon != None:
                                        Operation_material.objects.get_or_create(operation=operation_outcome, material=launch_operation_material.material_askon)
                                    if launch_operation_material.material != None:
                                        Operation_standard_prod.objects.get_or_create(operation=operation_outcome, material=launch_operation_material.material)

                                if Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item) == 0:
                                    raise Exception(f'Для : {stmp1} : {stmp2} ID={child_id} не задан ресурс.')

                                for launch_operation_resources in Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item):
                                    if launch_operation_resources.resource != None:
                                        Operation_resources.objects.get_or_create(operation=operation_outcome, resources=launch_operation_resources.resource)
                                    if launch_operation_resources.location != None:
                                        Operation_locations.objects.get_or_create(operation=operation_outcome, location=launch_operation_resources.location)

                                Operation_refs.objects.get_or_create(income=operation_income, outcome=operation_outcome)
                                operation_income = operation_outcome
                                cnt1 -= 1
                                if cnt1 == 0:
                                    items.append(dict(item_id=parent_id, first_operation=first_operation, last_operation=operation_outcome))
                        else:
                            raise Exception(f'Для : {stmp1} : {stmp2} не задано ни одной операции.')

                    sql_parents = f'''select child_id, stmp1, stmp2 from {mat_view_name} where parent_id = %s'''
                    cursor.execute(sql_parents, [parent_id])
                    parents_rows = cursor.fetchall()
                    for parents_row in parents_rows:
                        _child_id, stmp1, stmp2 = parents_row
                        _childs = [item for item in items if item.get('item_id') == _child_id]
                        if len(_childs) == 0:
                            raise Exception(f'Для {stmp1} : {stmp2} ID={_child_id} не найдена операция.')
                        elif len(_childs) > 1:
                            raise Exception(f'Для {stmp1} : {stmp2} ID={_child_id} неоднозначная операция.')

                        _child = _childs[0]

                        Operation_refs.objects.get_or_create(income=_child.get('last_operation'), outcome=first_operation)
                        deleted, _ = Operation_refs.objects.filter(income__isnull=True, outcome=first_operation).delete()
                    progress.setCntDone(cnt=cnt, id=id_progress1)
                    cnt += 1

                progress.close(id=id_progress1)
                cursor.execute(f'DROP MATERIALIZED VIEW {mat_view_name}')

            launch.props |= Launches.props.routing_maked
            launch.save()

    def clean_routing(data):
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from isc_common.auth.models.user import User

        logger.debug(f'data: {data}')

        launch_id = data.get('id')
        launch = Launches.objects.get(id=launch_id)

        cntAll = Operation_launches.objects.filter(launch=launch).count()

        if cntAll > 0:
            user = User.objects.get(id=data.get('user_id'))

            progress = ProgressStack(
                host=settings.WS_HOST,
                port=settings.WS_PORT,
                channel=f'common_{user.username}',
            )

            with transaction.atomic():
                id_progress1 = f'launch_{launch.id}_{user.id}'

                demand_str = f'<h3>Очистка данных Маршрутизации/Расписания : Запуск № {launch.code} от {DateToStr(launch.date)}</h3>'


                launch.props &= ~Launches.props.routing_maked
                launch.save()

                progress.show(
                    title=f'<b>Обработано товарных позиций</b>',
                    label_contents=demand_str,
                    cntAll=Operation_launches.objects.filter(launch=launch).count(),
                    id=id_progress1
                )

                cnt = 0

                for operation_launches in Operation_launches.objects.filter(launch=launch):
                    operation_launches.operation.delete()
                    progress.setCntDone(cnt=cnt, id=id_progress1)
                    cnt += 1

                progress.close(id=id_progress1)
        return model_to_dict(launch)


    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'opertype_id': record.opertype.id,
            'opertype_full_name': record.opertype.full_name,
            'value': record.value,
        }
        return res

    def get_queryset(self):
        return OperationsQuerySet(self.model, using=self._db)


class Operations(AuditModel):
    opertype = ForeignKeyProtect(Operation_types)
    status = ForeignKeyProtect(Status_operation_types)

    objects = OperationsManager()

    def __str__(self):
        return f"ID:{self.id}, opertype: [{self.opertype}], status: [{self.status}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Опреации учавствующие в планировании'



import logging

from django.core.management import BaseCommand

from kaf_pas.planing.models.launch_item_refs_timing import Launch_item_refs_timingManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        data = {'id': 15, 'code': '1', 'name': None, 'date': '2019-12-04T00:00:00.000', 'description': None, 'parent_id': None, 'demand_id': 34, 'demand__code': 'Тестовый №2', 'demand__date': '2019-10-31T00:00:00.000', 'status_id': 3, 'status__code': 'formirovanie', 'status__name': 'Формирование',
                'qty': 2, 'isFolder': False, 'editing': True, 'deliting': True, 'user_id': 2}
        Launch_item_refs_timingManager.make_timing(data=data)



# Generated by Django 2.2.2 on 2019-06-16 04:50

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0106_auto_20190614_1947'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 14:49

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0140_auto_20191204_1449'),
        ('ckk', '0162_locations_calendar'),
        ('planing', '0018_auto_20191204_0919'),
    ]

    operations = [
        migrations.CreateModel(
            name='Resource_loading',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('order', models.PositiveIntegerField()),
                ('launch_operations_item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Launch_operations_item')),
                ('resource', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Resource')),
            ],
            options={
                'verbose_name': 'Загрузка ресурсов',
            },
        ),
        migrations.CreateModel(
            name='Location_loading',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('order', models.PositiveIntegerField()),
                ('launch_operations_item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Launch_operations_item')),
                ('location', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations')),
            ],
            options={
                'verbose_name': 'Закрузка мест производства',
            },
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-29 10:22

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0028_auto_20190528_2223'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='code',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AddField(
            model_name='operations',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operations'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.uploades_documents import Uploades_documents, Uploades_documentsManager
from kaf_pas.kd.models.uploades_documents_view import Uploades_documents_view, Uploades_documentsViewManager


@JsonResponseWithException()
def Uploades_documents_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Uploades_documents_view.objects.
                select_related('document', 'upload').
                get_range_rows1(
                request=request,
                function=Uploades_documentsViewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_documents_Add(request):
    return JsonResponse(DSResponseAdd(data=Uploades_documents.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_documents_Update(request):
    return JsonResponse(DSResponseUpdate(data=Uploades_documents.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_documents_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Uploades_documents.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_documents_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Uploades_documents.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_documents_Info(request):
    return JsonResponse(DSResponse(request=request, data=Uploades_documents_view.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_documents_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Uploades_documents.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



import logging

from bitfield import BitField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from kaf_pas.ckk.models.item import Item

logger = logging.getLogger(__name__)


class Item_refsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def update(self, **kwargs):
        return super().update(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        return super().get_or_create(defaults, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        return super().update_or_create(defaults, **kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_refsManager(AuditManager):

    @staticmethod
    def props():
        return BitField(flags=(
        ('relevant', 'Актуальность'),  # 1
    ), default=1, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Item_refsQuerySet(self.model, using=self._db)


class Item_refs(AuditModel):
    child = ForeignKeyProtect(Item, related_name='child')
    parent = ForeignKeyProtect(Item, related_name='parent', blank=True, null=True)

    props = Item_refsManager.props()

    objects = TreeAuditModelManager()

    def __str__(self):
        return f'\nID={self.id} child=[{self.child}], parent=[{self.parent}]'

    def save(self, *args, **kwargs):
        if self.parent:
            if self.parent.id == self.child.id:
                raise Exception(f'Attempt to record with parent.id ({self.parent.id}) == child.id ({self.child.id})')
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = 'Item_refs'
        unique_together = ('child', 'parent')



import logging

from django.db.models import PositiveIntegerField

from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.planing.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_levelQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operation_levelManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_levelQuerySet(self.model, using=self._db)


class Operation_level(AuditModel):
    operation = ForeignKeyCascade(Operations, related_name='planing_operation_lev')
    level = PositiveIntegerField()

    objects = Operation_levelManager()

    def __str__(self):
        return f"ID:{self.id}, level: {self.level}"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Уровень вложенности операции'



import logging

from django.db.models import Model, PositiveIntegerField

from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Document_attr_cross(Model):
    document = ForeignKeyCascade(Documents)
    attribute = ForeignKeyCascade(Document_attributes)
    position_in_document = PositiveIntegerField()

    section = CodeField(verbose_name="Раздел", null=True, blank=True)
    subsection = CodeField(verbose_name="Подраздел", null=True, blank=True)

    def save(self, *args, **kwargs):
        if self.attribute.attr_type.code == 'STMP_1' or self.attribute.attr_type.code == 'STMP_2':
            try:
                Document_attr_cross.objects.get(attribute=self.attribute, document=self.document)
                raise Exception(f'Попытка записи дубликата: {self.attribute}')
            except Document_attr_cross.DoesNotExist:
                ...
        super().save(*args, **kwargs)

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('document', 'position_in_document'),)



from django.db import transaction
from django.forms import model_to_dict

from isc_common import delAttr, setAttr, getAttr
from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.combineItems import CombineItems
from kaf_pas.ckk.models.copyItems import CopyItems
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_line import Item_line
from kaf_pas.ckk.models.item_operations_view import Item_operations_view, Item_operations_viewManager
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.ckk.models.item_view import Item_view, Item_viewManager


def Item_view_query():
    query = Item_view.objects.exclude(id=3169551)
    return query


@JsonResponseWithException()
def Item_view_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_view_query().
                get_range_rows1(
                request=request,
                function=Item_viewManager.getRecord,
                distinct_field_names=('id',)
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Fetch1(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_operations_view.objects.
                exclude(id=3169551).
                get_range_rows1(
                request=request,
                function=Item_operations_viewManager.getRecord,
                distinct_field_names=('id',)
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Add(request):
    return JsonResponse(DSResponseAdd(data=Item.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Update(request):
    return JsonResponse(DSResponseUpdate(data=Item.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Info(request):
    return JsonResponse(DSResponse(request=request, data=Item_view_query().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_view_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Item_view.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def copyItems(request):
    return JsonResponse(CopyItems(request).response)


@JsonResponseWithException(printing=False)
def combineItems(request):
    return JsonResponse(CombineItems(request).response)


@JsonResponseWithException()
def Item_view_CopyBlockItems(request):
    _request = DSRequest(request=request)
    source = _request.json.get('source')
    destination = _request.json.get('destination')

    if isinstance(source, dict) and isinstance(destination, dict):
        srecords = source.get('records')
        drecord = destination.get('record')

        if isinstance(srecords, list) and isinstance(drecord, dict):
            with transaction.atomic():
                for srecord in srecords:
                    item_refs, created = Item_refs.objects.get_or_create(parent_id=drecord.get('id'), child_id=srecord.get('child_id'))
                    item_line = Item_line.objects.get(id=srecord.get('id'))
                    dict_item_line = model_to_dict(item_line)
                    delAttr(dict_item_line, 'id')
                    setAttr(dict_item_line, 'parent_id', drecord.get('id'))
                    delAttr(dict_item_line, 'parent')
                    setAttr(dict_item_line, 'child_id', getAttr(dict_item_line, 'child'))
                    delAttr(dict_item_line, 'child')

                    setAttr(dict_item_line, 'SPC_CLM_FORMAT_id', getAttr(dict_item_line, 'SPC_CLM_FORMAT'))
                    delAttr(dict_item_line, 'SPC_CLM_FORMAT')
                    setAttr(dict_item_line, 'SPC_CLM_ZONE_id', getAttr(dict_item_line, 'SPC_CLM_ZONE'))
                    delAttr(dict_item_line, 'SPC_CLM_ZONE')
                    setAttr(dict_item_line, 'SPC_CLM_POS_id', getAttr(dict_item_line, 'SPC_CLM_POS'))
                    delAttr(dict_item_line, 'SPC_CLM_POS')
                    setAttr(dict_item_line, 'SPC_CLM_MARK_id', getAttr(dict_item_line, 'SPC_CLM_MARK'))
                    delAttr(dict_item_line, 'SPC_CLM_MARK')
                    setAttr(dict_item_line, 'SPC_CLM_NAME_id', getAttr(dict_item_line, 'SPC_CLM_NAME'))
                    delAttr(dict_item_line, 'SPC_CLM_NAME')
                    setAttr(dict_item_line, 'SPC_CLM_COUNT_id', getAttr(dict_item_line, 'SPC_CLM_COUNT'))
                    delAttr(dict_item_line, 'SPC_CLM_COUNT')
                    setAttr(dict_item_line, 'SPC_CLM_NOTE_id', getAttr(dict_item_line, 'SPC_CLM_NOTE'))
                    delAttr(dict_item_line, 'SPC_CLM_NOTE')
                    setAttr(dict_item_line, 'SPC_CLM_MASSA_id', getAttr(dict_item_line, 'SPC_CLM_MASSA'))
                    delAttr(dict_item_line, 'SPC_CLM_MASSA')
                    setAttr(dict_item_line, 'SPC_CLM_MATERIAL_id', getAttr(dict_item_line, 'SPC_CLM_MATERIAL'))
                    delAttr(dict_item_line, 'SPC_CLM_MATERIAL')
                    setAttr(dict_item_line, 'SPC_CLM_USER_id', getAttr(dict_item_line, 'SPC_CLM_USER'))
                    delAttr(dict_item_line, 'SPC_CLM_USER')
                    setAttr(dict_item_line, 'SPC_CLM_KOD_id', getAttr(dict_item_line, 'SPC_CLM_KOD'))
                    delAttr(dict_item_line, 'SPC_CLM_KOD')
                    setAttr(dict_item_line, 'SPC_CLM_FACTORY_id', getAttr(dict_item_line, 'SPC_CLM_FACTORY'))
                    delAttr(dict_item_line, 'SPC_CLM_FACTORY')
                    parent_id = getAttr(dict_item_line, 'parent_id')
                    delAttr(dict_item_line, 'parent_id')
                    child_id = getAttr(dict_item_line, 'child_id')
                    delAttr(dict_item_line, 'child_id')
                    item_line, created = Item_line.objects.get_or_create(parent_id=parent_id, child_id=child_id, defaults=dict_item_line)

    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.document_attributes_view import Document_attributes_view

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Перенос атрибутов КД"

    def handle(self, *args, **options):

        logger.info(self.help)

        self.pbar = tqdm(total=Document_attributes_view.objects.filter().count())

        section_attr_id, _ = Attr_type.objects.get_or_create(code='section', name='Секция')
        sub_section_attr_id, _ = Attr_type.objects.get_or_create(code='subsection', name='Подсекция')

        c = 0
        u = 0
        with transaction.atomic():
            document_id = None

            for attribute in Document_attributes_view.objects.filter():
                if document_id != attribute.document.id:
                    document_id =  attribute.document.id
                    position_in_document = 0

                attr, created = Document_attributes.objects.get_or_create(
                    attr_type=attribute.attr_type,
                    value_str=attribute.value_str,
                )

                if created:
                    c += 1
                    logger.debug(f'{attr} been created ({c} times)')
                else:
                    u += 1
                    logger.debug(f'{attr} been used ({u} times)')

                Document_attr_cross.objects.create(
                    document=attribute.document,
                    section=attribute.section,
                    subsection=attribute.subsection,
                    position_in_document=position_in_document,
                    attribute=attr
                )

                position_in_document += 1

                if self.pbar:
                    self.pbar.update(1)

        if self.pbar:
            self.pbar.close()

    logger.info("Перенос выполнен.")
    # </editor-fold>

# with transaction.atomic():



import datetime
import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction
from django.db.models import DateTimeField, PositiveIntegerField, F

from isc_common import getAttr, setAttr
from isc_common.datetime import DateToStr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet
from isc_common.number import StrToNumber, DelProps
from isc_common.ws.progressStack import ProgressStack
from kaf_pas.production.models import SPC_CLM_COUNT_ATTR
from kaf_pas.production.models.status_launch import Status_launch
from kaf_pas.sales.models.demand import Demand
from kaf_pas.sales.models.demand_view import Demand_view

logger = logging.getLogger(__name__)


class LaunchesQuerySet(BaseRefQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class LaunchesManager(BaseRefManager):

    @staticmethod
    def get_count(item_line, qty):
        if qty == 1:
            return item_line.SPC_CLM_COUNT

        count = StrToNumber(item_line.SPC_CLM_COUNT.value_str if item_line.SPC_CLM_COUNT else None)
        SPC_CLM_COUNT = None
        if count:
            count *= qty
            from kaf_pas.kd.models.document_attributes import Document_attributes
            SPC_CLM_COUNT, created = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str=str(count))
            if created and isinstance(count, int):
                SPC_CLM_COUNT.value_int = count
                SPC_CLM_COUNT.save()

        return SPC_CLM_COUNT

    @staticmethod
    def make_launch(data):
        logger.debug(f'data: {data}')

        from isc_common.auth.models.user import User
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.production.models.launch_item_line import Launch_item_line
        from kaf_pas.production.models.launch_item_refs import Launch_item_refs
        from kaf_pas.production.models.launch_operations_item import Launch_operations_item
        from kaf_pas.production.models.operation_material import Operation_material
        from kaf_pas.production.models.operation_resources import Operation_resources
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.production.models.launch_operation_material import Launch_operations_material
        from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources

        demand_id = getAttr(data, 'demand_id')
        qty = getAttr(data, 'qty')
        code = getAttr(data, 'code')
        description = getAttr(data, 'description')
        date = getAttr(data, 'date')
        date = datetime.datetime.strptime(date, '%Y-%m-%d')
        user = User.objects.get(id=getAttr(data, 'user_id'))
        status, _ = Status_launch.objects.update_or_create(name='Формирование', editing=False, deliting=False)

        with transaction.atomic():
            for demand in Demand.objects.select_for_update().filter(id=demand_id):
                tail_qty = Demand_view.objects.get(id=demand_id).tail_qty
                if tail_qty < qty:
                    raise Exception(f'Затребованное количестово для запуска ({qty}), превышает возможное к запуску ({tail_qty})')

                launch = Launches.objects.create(
                    code=code,
                    date=date,
                    demand_id=demand_id,
                    description=description,
                    qty=qty,
                    status=status
                )

                progress = ProgressStack(
                    host=settings.WS_HOST,
                    port=settings.WS_PORT,
                    channel=f'common_{user.username}',
                )

                id_progress = f'launch_{launch.id}_{user.id}'
                demand_str = f'<h3>Формирование запуска: Заказ № {demand.code} от {DateToStr(demand.date)}</h3>'

                cntAll = Item_refs.objects.get_descendants_count(id=demand.precent_item.item.id, where_clause='where is_bit_on(props::integer, 0) = true')
                # logger.debug(f'cntAll: {cntAll}')
                progress.show(
                    title=f'<b>Обработано товарных позиций</b>',
                    label_contents=demand_str,
                    cntAll=cntAll,
                    id=id_progress
                )

                cnt = 0
                Item_refs.objects.filter(child=F('parent')).delete()

                for item_ref in Item_refs.objects.get_descendants(id=demand.precent_item.item.id, where_clause='where is_bit_on(props::integer, 0) = true'):

                    launch_item_refs, _ = Launch_item_refs.objects.get_or_create(
                        child=item_ref.child,
                        parent=item_ref.parent if item_ref.level != 1 else None,
                        item_refs=item_ref,
                        launch=launch
                    )

                    for item_line in Item_line.objects.filter(parent=item_ref.parent, child=item_ref.child):
                        # logger.debug(item_ref)

                        Launch_item_line.objects.get_or_create(
                            child=item_line.child,
                            parent=item_line.parent,
                            item_line=item_line,
                            launch=launch,

                            SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
                            SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
                            SPC_CLM_POS=item_line.SPC_CLM_POS,
                            SPC_CLM_MARK=item_line.SPC_CLM_MARK,
                            SPC_CLM_NAME=item_line.SPC_CLM_NAME,
                            SPC_CLM_COUNT=LaunchesManager.get_count(item_line, qty),
                            SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
                            SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
                            SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
                            SPC_CLM_USER=item_line.SPC_CLM_USER,
                            SPC_CLM_KOD=item_line.SPC_CLM_KOD,
                            SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
                            section=item_line.section,
                            section_num=item_line.section_num,
                            subsection=item_line.subsection,
                        )

                        # logger.debug(item_line)

                    def rec_operations_data(item):
                        for operations_item in Operations_item.objects.filter(item=item):
                            launch_operations_item, _ = Launch_operations_item.objects.get_or_create(
                                description=operations_item.description,
                                ed_izm=operations_item.ed_izm,
                                item=operations_item.item,
                                launch=launch,
                                num=operations_item.num,
                                operation=operations_item.operation,
                                operationitem=operations_item,
                                qty=operations_item.qty,
                            )
                            for operation_material in Operation_material.objects.filter(operationitem=operations_item):
                                Launch_operations_material.objects.get_or_create(
                                    operation_material=operation_material,
                                    launch_operationitem=launch_operations_item,
                                    material=operation_material.material,
                                    material_askon=operation_material.material_askon,
                                    edizm=operation_material.edizm,
                                    qty=operation_material.qty,
                                )
                            for operation_resources in Operation_resources.objects.filter(operationitem=operations_item):
                                Launch_operation_resources.objects.get_or_create(
                                    operation_resources=operation_resources,
                                    launch_operationitem=launch_operations_item,
                                    resource=operation_resources.resource,
                                    location=operation_resources.location,
                                    q_paralel=operation_resources.q_paralel,
                                )

                    if item_ref.parent:
                        rec_operations_data(item_ref.parent)
                    rec_operations_data(item_ref.child)

                    progress.setCntDone(cnt, id_progress)
                    cnt += 1

                progress.close(id=id_progress)
                settings.EVENT_STACK.EVENTS_PRODUCTION_MAKE_LAUNCH.send_message(f'Выполнено формирование запуска  <h3>{launch.code} от {launch.date}</h3><p/>')
        return data

    # @staticmethod
    # def update_launch(data):
    #     logger.debug(f'data: {data}')
    #
    #     from isc_common.auth.models.user import User
    #     from kaf_pas.ckk.models.item_line import Item_line
    #     from kaf_pas.ckk.models.item_refs import Item_refs
    #     from kaf_pas.production.models.launch_item_line import Launch_item_line
    #     from kaf_pas.production.models.launch_item_refs import Launch_item_refs
    #     from kaf_pas.production.models.launch_operations_item import Launch_operations_item
    #     from kaf_pas.production.models.operation_material import Operation_material
    #     from kaf_pas.production.models.operation_resources import Operation_resources
    #     from kaf_pas.production.models.operations_item import Operations_item
    #     from kaf_pas.production.models.launch_operation_material import Launch_operations_material
    #     from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
    #
    #     id = data.get('id')
    #     demand_id = data.get('demand_id')
    #     qty = data.get('qty')
    #     code = data.get('code')
    #     description = data.get('description')
    #     date = data.get('date')
    #     date = datetime.datetime.strptime(date.replace('T00:00:00.000', ''), '%Y-%m-%d')
    #     user = User.objects.get(id=getAttr(data, 'user_id'))
    #     status, _ = Status_launch.objects.update_or_create(name='Переформирование', editing=False, deliting=False)
    #
    #     def rec_operations_data(item, launch_item_refs, item_message):
    #         for operations_item in Operations_item.objects.filter(item=item):
    #
    #             try:
    #                 launch_operations_item = Launch_operations_item.objects.get(
    #                     description=operations_item.description,
    #                     ed_izm=operations_item.ed_izm,
    #                     item=operations_item.item,
    #                     launch=parent_launch,
    #                     num=operations_item.num,
    #                     operation=operations_item.operation,
    #                     operationitem=operations_item,
    #                 )
    #                 if launch_operations_item.qty != operations_item.qty:
    #                     _launch_operations_item, _ = Launch_operations_item.objects.get_or_create(
    #                         description=operations_item.description,
    #                         ed_izm=operations_item.ed_izm,
    #                         item=operations_item.item,
    #                         launch=launch,
    #                         num=operations_item.num,
    #                         operation=operations_item.operation,
    #                         operationitem=operations_item,
    #                         parent=launch_operations_item,
    #                         qty=launch_operations_item.qty,
    #                     )
    #                     Launch_operations_item.objects.get_or_create(
    #                         description=operations_item.description,
    #                         ed_izm=operations_item.ed_izm,
    #                         item=operations_item.item,
    #                         launch=parent_launch,
    #                         num=operations_item.num,
    #                         operation=operations_item.operation,
    #                         operationitem=operations_item,
    #                         parent=_launch_operations_item,
    #                         qty=operations_item.qty,
    #                     )
    #                     launch_operations_item.props &= ~Launch_operations_item.props.enabled
    #                     item_message.append(f'Для операции {launch_operations_item} изменена длительность.')
    #                     launch_item_refs.props |= Launch_item_refs.props.edited
    #                     launch_item_refs.save()
    #                 else:
    #                     launch_operations_item.props |= Launch_operations_item.props.refreshed
    #                 launch_operations_item.save()
    #
    #                 for operation_material in Operation_material.objects.filter(operationitem=operations_item):
    #                     try:
    #                         launch_operations_material = Launch_operations_material.objects.get(
    #                             operation_material=operation_material,
    #                             launch_operationitem=launch_operations_item,
    #                             material=operation_material.material,
    #                             material_askon=operation_material.material_askon,
    #                             edizm=operation_material.edizm,
    #                             qty=operation_material.qty,
    #                         )
    #
    #                         if launch_operations_material.qty != operation_material.qty:
    #                             _launch_operations_material, _ = Launch_operations_material.objects.get_or_create(
    #                                 operation_material=operation_material,
    #                                 launch_operationitem=launch_operations_item,
    #                                 material=operation_material.material,
    #                                 material_askon=operation_material.material_askon,
    #                                 edizm=operation_material.edizm,
    #                                 qty=launch_operations_material.qty,
    #                                 parent=launch_operations_material,
    #                             )
    #
    #                             Launch_operations_material.objects.get_or_create(
    #                                 operation_material=operation_material,
    #                                 launch_operationitem=launch_operations_item,
    #                                 material=operation_material.material,
    #                                 material_askon=operation_material.material_askon,
    #                                 edizm=operation_material.edizm,
    #                                 qty=launch_operations_material.qty,
    #                                 parent=_launch_operations_material,
    #                             )
    #
    #                             item_message.append(f'Для материала {launch_operations_material} изменено количество.')
    #
    #                             launch_operations_material.props &= ~Launch_operations_material.props.enabled
    #                             launch_item_refs.props |= Launch_item_refs.props.edited
    #                             launch_item_refs.save()
    #                         else:
    #                             launch_operations_material.props |= Launch_operations_material.props.refreshed
    #                         launch_operations_material.save()
    #
    #                     except Launch_operations_material.DoesNotExist:
    #                         Launch_operations_material.objects.get_or_create(
    #                             operation_material=operation_material,
    #                             launch_operationitem=launch_operations_item,
    #                             material=operation_material.material,
    #                             material_askon=operation_material.material_askon,
    #                             edizm=operation_material.edizm,
    #                             qty=operation_material.qty,
    #                         )
    #
    #                 for operation_resources in Operation_resources.objects.filter(operationitem=operations_item):
    #                     try:
    #                         launch_operation_resources = Launch_operation_resources.objects.get(
    #                             operation_resources=operation_resources,
    #                             launch_operationitem=launch_operations_item,
    #                             resource=operation_resources.resource,
    #                             location=operation_resources.location,
    #                         )
    #                         launch_operation_resources.props |= Launch_operation_resources.props.refreshed
    #                         launch_operation_resources.save()
    #                     except Launch_operation_resources.DoesNotExist:
    #                         Launch_operation_resources.objects.get_or_create(
    #                             operation_resources=operation_resources,
    #                             launch_operationitem=launch_operations_item,
    #                             resource=operation_resources.resource,
    #                             location=operation_resources.location,
    #                             q_paralel=operation_resources.q_paralel,
    #                         )
    #
    #             except Launch_operations_item.DoesNotExist:
    #                 launch_operations_item, created = Launch_operations_item.objects.get_or_create(
    #                     description=operations_item.description,
    #                     ed_izm=operations_item.ed_izm,
    #                     item=operations_item.item,
    #                     launch=launch,
    #                     num=operations_item.num,
    #                     operation=operations_item.operation,
    #                     operationitem=operations_item,
    #                     qty=operations_item.qty,
    #                 )
    #                 item_message.append(f'Добавлена операция {launch_operations_item}.')
    #                 for operation_material in Operation_material.objects.filter(operationitem=operations_item):
    #                     Launch_operations_material.objects.get_or_create(
    #                         operation_material=operation_material,
    #                         launch_operationitem=launch_operations_item,
    #                         material=operation_material.material,
    #                         material_askon=operation_material.material_askon,
    #                         edizm=operation_material.edizm,
    #                         qty=operation_material.qty,
    #                     )
    #                     item_message.append(f'Добавлен материал {operation_material}.')
    #                 for operation_resources in Operation_resources.objects.filter(operationitem=operations_item):
    #                     Launch_operation_resources.objects.get_or_create(
    #                         operation_resources=operation_resources,
    #                         launch_operationitem=launch_operations_item,
    #                         resource=operation_resources.resource,
    #                         location=operation_resources.location,
    #                         q_paralel=operation_resources.q_paralel,
    #                     )
    #                     item_message.append(f'Добавлен ресурс {operation_resources}.')
    #
    #                 launch_item_refs.props |= Launch_item_refs.props.edited
    #                 launch_item_refs.save()
    #
    #         # logger.debug(f'not_rfreshed_or_edited_qty: {get_not_rfreshed_or_edited_qty()}')
    #         for launch_operations_item in Launch_operations_item.objects. \
    #                 filter(item=item, launch=parent_launch). \
    #                 exclude(props=Launch_operations_item.props.refreshed):
    #             launch_operations_item.props &= ~Launch_operations_item.props.enabled
    #             launch_operations_item.disabled_launch = launch
    #             launch_operations_item.save()
    #
    #     def append_line(item_line, SPC_CLM_COUNT):
    #         launch_item_line, _ = Launch_item_line.objects.get_or_create(
    #             child=item_line.child,
    #             parent=item_line.parent,
    #             item_line=item_line,
    #             launch=launch,
    #
    #             SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
    #             SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
    #             SPC_CLM_POS=item_line.SPC_CLM_POS,
    #             SPC_CLM_MARK=item_line.SPC_CLM_MARK,
    #             SPC_CLM_NAME=item_line.SPC_CLM_NAME,
    #             SPC_CLM_COUNT=SPC_CLM_COUNT,
    #             SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
    #             SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
    #             SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
    #             SPC_CLM_USER=item_line.SPC_CLM_USER,
    #             SPC_CLM_KOD=item_line.SPC_CLM_KOD,
    #             SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
    #             section=item_line.section,
    #             section_num=item_line.section_num,
    #             subsection=item_line.subsection,
    #         )
    #
    #         Launch_item_line.objects.get_or_create(
    #             child=item_line.child,
    #             parent=item_line.parent,
    #             added_parent=launch_item_line,
    #             item_line=item_line,
    #             launch=parent_launch,
    #
    #             SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
    #             SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
    #             SPC_CLM_POS=item_line.SPC_CLM_POS,
    #             SPC_CLM_MARK=item_line.SPC_CLM_MARK,
    #             SPC_CLM_NAME=item_line.SPC_CLM_NAME,
    #             SPC_CLM_COUNT=SPC_CLM_COUNT,
    #             SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
    #             SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
    #             SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
    #             SPC_CLM_USER=item_line.SPC_CLM_USER,
    #             SPC_CLM_KOD=item_line.SPC_CLM_KOD,
    #             SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
    #             section=item_line.section,
    #             section_num=item_line.section_num,
    #             subsection=item_line.subsection,
    #             editing=False,
    #             deliting=False,
    #         )
    #
    #     with transaction.atomic():
    #         for parent_launch in Launches.objects.select_for_update().filter(id=id):
    #             for demand in Demand.objects.select_for_update().filter(id=demand_id):
    #                 tail_qty = Demand_view.objects.get(id=demand_id).tail_qty + parent_launch.qty
    #                 if tail_qty < qty:
    #                     raise Exception(f'Затребованное количестово для переформирования ({qty}), превышает возможное к запуску ({tail_qty})')
    #
    #                 launch, _ = Launches.objects.get_or_create(
    #                     code=code,
    #                     date=date,
    #                     demand_id=demand_id,
    #                     description=description,
    #                     qty=qty,
    #                     status=status,
    #                     parent=parent_launch,
    #                     editing=False
    #                 )
    #
    #                 progress = ProgressStack(
    #                     host=settings.WS_HOST,
    #                     port=settings.WS_PORT,
    #                     channel=f'common_{user.username}',
    #                 )
    #
    #                 id_progress1 = f'launch_{launch.id}_{user.id}'
    #                 demand_str = f'<h3>Переформирование запуска (этап-I): № {parent_launch.code} от {DateToStr(parent_launch.date)}</h3>'
    #
    #                 cntAll = Item_refs.objects.get_descendants_count(id=demand.precent_item.item.id)
    #                 progress.show(
    #                     title=f'<b>Обработано товарных позиций</b>',
    #                     label_contents=demand_str,
    #                     cntAll=cntAll,
    #                     id=id_progress1
    #                 )
    #
    #                 cnt = 0
    #                 Item_refs.objects.filter(child=F('parent')).delete()
    #
    #                 for item_ref in Item_refs.objects.get_descendants(id=demand.precent_item.item.id):
    #                     item_message = []
    #                     launch_item_refs, created = Launch_item_refs.objects.get_or_create(
    #                         child=item_ref.child,
    #                         parent=item_ref.parent if item_ref.level != 1 else None,
    #                         item_refs=item_ref,
    #                         launch=parent_launch,
    #                         defaults=dict(
    #                             props=Launch_item_refs.props.enabled | Launch_item_refs.props.added,
    #                             added_launch=launch
    #                         )
    #                     )
    #
    #                     if created:
    #                         launch_item_refs, _ = Launch_item_refs.objects.get_or_create(
    #                             child=item_ref.child,
    #                             parent=item_ref.parent if item_ref.level != 1 else None,
    #                             item_refs=item_ref,
    #                             launch=launch,
    #                         )
    #
    #                         item_message.append(f'Товарная позиция: {launch_item_refs.child.item_name} добавлена в спецификациию, как ранее не существовавшая.')
    #
    #                         for item_line in Item_line.objects.filter(parent=launch_item_refs.parent, child=launch_item_refs.child):
    #                             append_line(item_line, LaunchesManager.get_count(item_line, qty))
    #
    #                     else:
    #                         launch_item_refs, _ = Launch_item_refs.objects.get_or_create(
    #                             child=item_ref.child,
    #                             parent=item_ref.parent if item_ref.level != 1 else None,
    #                             item_refs=item_ref,
    #                             launch=launch,
    #                         )
    #
    #                         for item_line in Item_line.objects.filter(parent=launch_item_refs.parent, child=launch_item_refs.child):
    #                             try:
    #                                 launch_item_line = Launch_item_line.objects.get(
    #                                     child=item_line.child,
    #                                     parent=item_line.parent,
    #                                     item_line=item_line,
    #                                     launch=parent_launch,
    #                                 )
    #
    #                                 if item_line.SPC_CLM_FORMAT != launch_item_line.SPC_CLM_FORMAT or \
    #                                         item_line.SPC_CLM_ZONE != launch_item_line.SPC_CLM_ZONE or \
    #                                         item_line.SPC_CLM_POS != launch_item_line.SPC_CLM_POS or \
    #                                         item_line.SPC_CLM_MARK != launch_item_line.SPC_CLM_MARK or \
    #                                         item_line.SPC_CLM_NAME != launch_item_line.SPC_CLM_NAME or \
    #                                         LaunchesManager.get_count(item_line, qty) != launch_item_line.SPC_CLM_COUNT or \
    #                                         item_line.SPC_CLM_NOTE != launch_item_line.SPC_CLM_NOTE or \
    #                                         item_line.SPC_CLM_MASSA != launch_item_line.SPC_CLM_MASSA or \
    #                                         item_line.SPC_CLM_MATERIAL != launch_item_line.SPC_CLM_MATERIAL or \
    #                                         item_line.SPC_CLM_USER != launch_item_line.SPC_CLM_USER or \
    #                                         item_line.SPC_CLM_KOD != launch_item_line.SPC_CLM_KOD or \
    #                                         item_line.SPC_CLM_FACTORY != launch_item_line.SPC_CLM_FACTORY or \
    #                                         item_line.section != launch_item_line.section or \
    #                                         item_line.section_num != launch_item_line.section_num or \
    #                                         item_line.subsection != launch_item_line.subsection:
    #                                     launch_item_line.props &= ~Launch_item_line.props.enabled
    #                                     launch_item_line.save()
    #
    #                                     item_message.append(f'Изменена строка детелизации : {launch_item_line.item_name}.')
    #
    #                                     append_line(item_line, LaunchesManager.get_count(item_line, qty))
    #                                     launch_item_refs.props |= Launch_item_refs.props.edited
    #                                     launch_item_refs.save()
    #                                 else:
    #                                     pass
    #
    #                             except Launch_item_line.DoesNotExist:
    #                                 item_message.append(f'Cтрока детелизации : {item_line.child.item_name} добавлена, как ранее не существовавшая.')
    #                                 append_line(item_line, LaunchesManager.get_count(item_line, qty))
    #
    #                     if item_ref.parent:
    #                         rec_operations_data(item=launch_item_refs.parent, launch_item_refs=launch_item_refs, item_message=item_message)
    #                     rec_operations_data(item=launch_item_refs.child, launch_item_refs=launch_item_refs, item_message=item_message)
    #
    #                     if ~launch_item_refs.props.edited:
    #                         launch_item_refs.delete()
    #
    #                     progress.setCntDone(cnt, id_progress1)
    #                     cnt += 1
    #
    #                 launch_item_refs = Launch_item_refs.objects.filter(launch=launch)
    #                 cntAll = launch_item_refs.count()
    #                 if cntAll == 0:
    #                     msg_str = f'Выполненое Переформирование запуска  <h3>{parent_launch.code} от {parent_launch.date}</h3> изменений не обнаружило.<p/>'
    #                     launch.delete()
    #                 else:
    #                     id_progress2 = f'launch_{launch.id}_{user.id}'
    #                     demand_str = f'<h3>Переформирование запуска (этап-II): № {parent_launch.code} от {DateToStr(parent_launch.date)}</h3>'
    #
    #                     progress.show(
    #                         title=f'<b>Обработано товарных позиций</b>',
    #                         label_contents=demand_str,
    #                         cntAll=cntAll,
    #                         id=id_progress2
    #                     )
    #
    #                     cnt = 0
    #                     for launch_item in launch_item_refs:
    #                         for _launch_item in Launch_item_refs.tree_objects.get_descendants(child_id=launch_item.child.id, parent_id=launch_item.parent.id, where_clause=f' where r.launch_id={launch.id}'):
    #                             print(_launch_item)
    #                         progress.setCntDone(cnt, id_progress2)
    #                         cnt += 1
    #                     progress.close(id=id_progress2)
    #                     msg_str = f'Выполнено Переформирование запуска  <h3>{parent_launch.code} от {parent_launch.date}</h3><p/>'
    #
    #             progress.close(id=id_progress1)
    #             settings.EVENT_STACK.EVENTS_PRODUCTION_RE_MAKE_LAUNCH.send_message(msg_str)
    #
    #     return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        setAttr(data, 'user_id', request.user_id)
        return LaunchesManager.make_launch(data)

    def reCreateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        setAttr(data, 'user_id', request.user_id)
        return LaunchesManager.update_launch(data)

    def deleteFromRequest(self, request, removed=None, ):
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.production.models.launch_item_line import Launch_item_line
        from kaf_pas.production.models.launch_item_refs import Launch_item_refs
        from kaf_pas.production.models.launch_operation_material import Launch_operations_material
        from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
        from kaf_pas.production.models.launch_operations_item import Launch_operations_item

        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        progress = ProgressStack(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{request.username}',
        )

        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    launch = Launches.objects.get(id=id)
                    id_progress = f'launch_{id}_{request.user_id}'
                    demand_str = f'<h3>Удаление запуска: № {launch.code} от {DateToStr(launch.date)}</h3>'

                    cntAll = Launch_operations_item.objects.filter(launch=id).count()
                    cnt = 0
                    # logger.debug(f'cntAll: {cntAll}')
                    progress.show(
                        title=f'<b>Обработано товарных позиций</b>',
                        label_contents=demand_str,
                        cntAll=cntAll,
                        id=id_progress
                    )

                    qty, _ = Launch_item_refs.objects.filter(launch_id=id).delete()
                    qty1, _ = Launch_item_line.objects.filter(launch_id=id).delete()
                    qty += qty1

                    for launch_operations_item in Launch_operations_item.objects.filter(launch=id):
                        qty1, _ = Launch_operations_material.objects.filter(launch_operationitem=launch_operations_item).delete()
                        qty += qty1
                        qty1, _ = Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item).delete()
                        qty += qty1
                        launch_operations_item.delete()
                        qty += 1
                        progress.setCntDone(cnt, id_progress)
                        cnt += 1

                    for operation_launches in Operation_launches.objects.filter(launch_id=id):
                        operation_launches.operation.delete()
                    qty1, _ = super().filter(id=id).delete()

                    progress.close(id=id_progress)
                    settings.EVENT_STACK.EVENTS_PRODUCTION_DELETE_LAUNCH.send_message(f'Выполнено удаление запуска  <h3>{launch.code} от {launch.date}</h3><p/>')
                res += qty + qty1
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'date': record.date,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,

            'demand_id': record.demand.id,
            'demand__code': record.demand.code,
            'demand__date': record.demand.date,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'qty': record.qty,
            'routing_maked': record.props.routing_maked,
            'props': record.props,

            'editing': record.editing,
            'deliting': record.deliting,
        }
        return DelProps(res)

    @staticmethod
    def props():
        return BitField(flags=(
        ('routing_maked', 'Маршрутизация выполнена'),  # 1
    ), default=0, db_index=True)

    def get_queryset(self):
        return LaunchesQuerySet(self.model, using=self._db)


class Launches(BaseRefHierarcy):
    date = DateTimeField()
    status = ForeignKeyProtect(Status_launch)
    demand = ForeignKeyProtect(Demand)
    qty = PositiveIntegerField()
    props = LaunchesManager.props()

    objects = LaunchesManager()

    def __str__(self):
        return f"ID:{self.id}, " \
               f"code: {self.code}, " \
               f"name: {self.name}, " \
               f"description: {self.description}, " \
               f"date: {self.date}, " \
               f"status: [{self.status}], " \
               f"demand: [{self.demand}], " \
               f"qty: {self.qty}"

    class Meta:
        verbose_name = 'Запуски'



import logging

from django.db import transaction
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.production.models.operations import Operations
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Operation_def_resourcesQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        if kwargs.get('resource'):
            setAttr(kwargs, 'resource_id', kwargs.get('resource').id)
            delAttr(kwargs, 'resource')

        if kwargs.get('location'):
            setAttr(kwargs, 'location_id', kwargs.get('location').id)
            delAttr(kwargs, 'location')

        if not kwargs.get('resource_id') and not kwargs.get('location_id'):
            raise Exception('Необходим хотябы один выпбранный параметр.')
        delAttr(kwargs, 'resource__full_name')
        delAttr(kwargs, 'location__full_name')

        resource_id = kwargs.get('resource_id')
        if resource_id:
            setAttr(kwargs, 'location_id', Resource.objects.get(id=resource_id).location.id)

        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operation_def_resourcesManager(CommonManagetWithLookUpFieldsManager):

    def allUpdateFromRequest(self, request):
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.production.models.operation_resources import Operation_resources

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        qty = 0

        with transaction.atomic():
            for operation_def_resource in super().filter(id__in=data.get('ids')).select_for_update():
                for operation_item in Operations_item.objects.filter(operation=operation_def_resource.operation):
                    deleted, _ = Operation_resources.objects.filter(operationitem=operation_item).delete()
                    logger.debug(f'Deleted: {deleted}')
                    operation_resource, _ = Operation_resources.objects.get_or_create(
                        operationitem=operation_item,
                        resource=operation_def_resource.resource,
                        location=operation_def_resource.location,
                    )
                    logger.debug(f'Created: {operation_resource}')
                qty += 1

        return qty

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        operation_id = _data.get('operation_id')

        res = []
        if isinstance(operation_id, list):
            delAttr(_data, 'operation_id')
            with transaction.atomic():
                for operation in operation_id:
                    setAttr(_data, 'operation_id', operation)
                    operation_resource = super().create(**_data)
                    _res = model_to_dict(operation_resource)
                    setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                    setAttr(_res, 'resource__full_name', operation_resource.resource.full_name if operation_resource.resource else None)
                    res.append(_res)
        else:
            operation_resource = super().create(**_data)
            _res = model_to_dict(operation_resource)
            setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
            setAttr(_res, 'resource__full_name', operation_resource.resource.full_name if operation_resource.resource else None)
            res.append(_res)
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operation_id': record.operation.id,
            'resource_id': record.resource.id if record.resource else None,
            'resource__code': record.resource.code if record.resource else None,
            'resource__name': record.resource.name if record.resource else None,
            'resource__full_name': record.resource.full_name if record.resource else None,
            'location_id': record.location.id if record.location else None,
            'location__code': record.location.code if record.location else None,
            'location__name': record.location.name if record.location else None,
            'location__full_name': record.location.full_name if record.location else None,
            'complex_name': record.complex_name,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_def_resourcesQuerySet(self.model, using=self._db)


class Operation_def_resources(AuditModel):
    operation = ForeignKeyCascade(Operations)
    resource = ForeignKeyProtect(Resource, null=True, blank=True)
    location = ForeignKeyProtect(Locations, null=True, blank=True)

    @property
    def complex_name(self):
        return f'{self.resource.location.full_name}{self.resource.full_name}' if self.resource else None

    objects = Operation_def_resourcesManager()

    def __str__(self):
        return f"ID: {self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'



import logging

from django.core.management import BaseCommand
from tqdm import tqdm

from kaf_pas.ckk.models.item import Item
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Удаление составов изделий"

    def handle(self, *args, **options):

        logger.info(self.help)

        self.pbar = tqdm(total=Documents.objects.filter(props=Documents.props.beenItemed).count())

        for spw in Documents.objects.filter(props=Documents.props.beenItemed):
            spw.props = Documents.props.relevant
            spw.save()

            if self.pbar:
                self.pbar.update(1)

        if self.pbar:
            self.pbar.close()

        self.pbar = tqdm(total=Item.objects.filter().count())

        for item in Item.objects.filter():
            item.delete()

            if self.pbar:
                self.pbar.update(1)

        if self.pbar:
            self.pbar.close()

        logger.info("Удаление выполнено.")
        # </editor-fold>

# with transaction.atomic():



import logging

from django.db.models import TextField

from crypto.models.crypto_file import Crypto_file, CryptoManager
from isc_common.fields.related import ForeignKeyProtect
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Documents_thumb10(Crypto_file):
    # Менять на cascade нельзя, потому как не происходит удаленеи файлов изображений при удалении документа
    document = ForeignKeyProtect(Documents, verbose_name='КД')
    path = TextField()

    objects = CryptoManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'JPEG варианты документов'



import logging

from django.core.management import BaseCommand
from django.db import transaction, connection

from kaf_pas.production.models.operations_item import Operations_item, Operations_itemManager
from kaf_pas.production.models.ready_2_launch import Ready_2_launchManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def handle(self, *args, **options):
        Operations_itemManager.refresh_num()



import logging

from bitfield import BitField

from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditManager
from isc_common.models.base_ref import BaseRef

logger = logging.getLogger(__name__)


class AttrManager(AuditManager):
    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "code": record.code,
            "name": record.name,
            "description": record.description,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_attr(self, code, name=None, default=None):
        try:
            if name:
                return super().get_queryset().get(code=code, name=name)
            else:
                return super().get_queryset().get(code=code)
        except Attr_type.DoesNotExist as e:
            if default:
                return default
            else:
                raise e


class Attr_type(BaseRef):
    code = CodeField(verbose_name="Код", db_index=True, unique=True)
    name = NameField(verbose_name="Наименование", db_index=True)
    props = BitField(flags=(
        ('relevant', 'Актуальность'),
    ), default=1, db_index=True)
    parent = ForeignKeyProtect("self", null=True, blank=True)

    relevent_label = "Действует"
    not_relevent_label = "Не действует"

    objects = AttrManager()

    def __str__(self):
        return f"ID={self.id}, code={self.code} name={self.name} props={self.props}"

    class Meta:
        verbose_name = 'Тип атрибута'



import logging

from django.core.management import BaseCommand
from django.db import transaction, connection
from django.db.models import ProtectedError
from tqdm import tqdm

from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_thumb import Documents_thumb
from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
from kaf_pas.kd.models.uploades_documents import Uploades_documents

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Удаление дубликатов чертежей"

    def handle(self, *args, **options):

        logger.info(self.help)

        sql_txt = '''select
                file_document,
                file_change_time,
                file_modification_time,
                path_id,
                deleted_at,
                count(*)
            from
                kd_documents
            group by
                file_document,
                file_change_time,
                path_id,
                deleted_at,
                file_modification_time
            having
                count(*) > 1'''

        with transaction.atomic():
            with connection.cursor() as cursor:
                cursor.execute(f'''select count(*)
                                   from ({sql_txt}) as a''')
                row = cursor.fetchone()
                self.pbar = tqdm(total=row[0])

                cursor.execute(sql_txt)
                row = cursor.fetchall()
                for p in row:
                    file_document = p[0]
                    logger.debug(f'\nDeleting: {file_document}')

                    for document in Documents.objects.filter(file_document=file_document):
                        Documents_thumb10.objects.filter(document=document).delete()
                        Documents_thumb.objects.filter(document=document).delete()
                        Uploades_documents.objects.filter(document=document).delete()
                        try:
                            res, _ = document.delete()
                            logger.debug(f'\n\nDFELETED {file_document}')
                        except ProtectedError:
                            logger.debug(f'\n\nNOT DFELETED {file_document}')

                    if self.pbar:
                        self.pbar.update(1)

                if self.pbar:
                    self.pbar.close()

        logger.info("Удаление выполнено")
